'''
Parsers package contains definitions for the 3 parsers used

@author:     Teodor G Nistor

@copyright:  2018 Teodor G Nistor

@license:    MIT License
'''

# Import parsers from subpackages using specific names
from beamr.parsers.document import parser as docParser
from beamr.parsers.slide import parser as slideParser
from beamr.parsers.image import parser as imageParser
