'''
Interpreters package contains classes of which instances are created for every
node in the document hierarchy.

@author:     Teodor G Nistor

@copyright:  2018 Teodor G Nistor

@license:    MIT License
'''

# Import interpreters from subpackages using specific names
from beamr.interpreters.textual import *
from beamr.interpreters.config import Config
from beamr.interpreters.hierarchical import *
