'''
Lexers package contains definitions for the 3 lexers used

@author:     Teodor G Nistor

@copyright:  2018 Teodor G Nistor

@license:    MIT License
'''

# Import lexers from subpackages using specific names
from beamr.lexers.document import lexer as docLexer
from beamr.lexers.slide import lexer as slideLexer
from beamr.lexers.image import lexer as imageLexer
