'''
Beamr - Minimal markup language for Beamer

Run the package as a module

@author:     Teodor G Nistor

@copyright:  2018 Teodor G Nistor

@license:    MIT License
'''

from beamr.cli import main
from sys import exit

exit(main())
